import { logger } from "../services";
import { getServiceToServiceToken, graphqlClient, missionManagementApiClient, usxDataBridgeClient } from "../api-clients";
import { LOG_PAYLOAD_ORDER_NUMBERS, SHOULD_PROCESS_ORDER_CHAINING } from "../conifg";
import { buildUsxOrderCreatePayload } from "./buildUSXOrderCreatePayload";

export const processOrderChainingEvent = async (data: any): Promise<void> => {
  const location = "processOrderChainingEvent";

  const { meta, payload } = JSON.parse(data.message.value?.toString());

  if (meta.intent !== "event-created" && meta.intent !== "event-upserted") {
    return;
  }

  const { bill_to, first_stop, last_stop, status, pickup_requirements, order_number: sourceOrderNumber } = payload;

  if (pickup_requirements !== "P") {
    logger.trace({
      location,
      message: "Skipping load because of pickup_requirements is not P"
    });
    return;
  }

  if (status === "FN") {
    logger.trace({
      location,
      message: "Skipping load because of status FN"
    });
    return;
  }

  if (LOG_PAYLOAD_ORDER_NUMBERS.includes(String(sourceOrderNumber))) {
    logger.trace({
      location,
      event_intent: meta.intent,
      payload
    });
  }

  // if source order's appointments are not set, do not process it.
  if (!first_stop.appointment_set || !last_stop.appointment_set) {
    return;
  }

  const token = await getServiceToServiceToken();
  missionManagementApiClient.setToken(token);

  // if there is already a created related load, do not create a new one.
  const existingRelatedLoadCreation = await missionManagementApiClient.searchRelatedLoadCreation({ sourceOrderNumber: String(sourceOrderNumber) });
  if (existingRelatedLoadCreation?.length) {
    return;
  }

  const laneRules = await missionManagementApiClient.searchLaneRule({
      billTo: bill_to,
      originCustomer: first_stop?.stop_customer,
      destinationCustomer: last_stop?.stop_customer
  });

  if (!laneRules || !laneRules.length) {
    return;
  }

  const canCreateMission = await graphqlClient.validateMission([String(sourceOrderNumber)]);
  if (!canCreateMission.valid) {
    logger.trace({
      location,
      message: `Skipping unmissionable order ${sourceOrderNumber}`,
      canCreateMission,
      payload
    });
    return;
  }

  const orderCreatePayload = buildUsxOrderCreatePayload(payload, laneRules[0].loadCreationAction);

  logger.trace({
    location,
    message: `Lane rule applicable order ${sourceOrderNumber}`,
    payload,
    laneRules,
    orderCreatePayload
  });

  let response, orderNumber;
  if (!SHOULD_PROCESS_ORDER_CHAINING) {
    return;
  }

  try {
    ({ response, orderNumber } = await usxDataBridgeClient.sendOrderCreate(orderCreatePayload));
  } catch (error) {
    logger.error({
      location,
      message: "Related Load Creation Failed",
      orderNumber: sourceOrderNumber,
      error
    });
    throw error;
  }

  const relatedLoadCreation = await missionManagementApiClient.createRelatedLoadCreation({
    sourceOrderNumber: String(sourceOrderNumber),
    relatedOrderNumber: orderNumber,
    loadCreationActionId: laneRules[0].loadCreationAction.id
  });

  logger.trace({
    location,
    message: "Related Load Creation Created",
    relatedLoadCreation,
    source: {
      billTo: bill_to,
      originCustomer: first_stop?.stop_customer,
      destinationCustomer: last_stop?.stop_customer
    }
  });

  return;
};
